; Use and free a button in QPAC 2's Button Frame for BASIC
; (C) 1996 Jochen Merz

	include win1_keys_err
	include win1_keys_qlv
	include win1_keys_thg
	include win1_keys_sys
	include win1_keys_qdos_sms
	include win1_keys_bv

	section sbext

	lea	proc_def,a1	; procedures and function table
	move.w	sb.inipr,a2
	jmp	(a2)		; initialise new procs and functions

proc_def
	dc.w	2		; one long procedure name
	dc.w	btframe_free-*
	dc.b	12,'BTFRAME_FREE '
	dc.w	0		; end of procedures
	dc.w	2		; two long function names
	dc.w	btframe_use-*
	dc.b	11,'BTFRAME_USE'
	dc.w	0		; end of functions

btframe_use
	move.w	sb.gtint,a2	; get integer parameters
	jsr	(a2)
	tst.l	d0
	bne.s	bt_use_ret	; error, return
	moveq	#err.ipar,d0	; assume invalid parameters
	subq.w	#2,d3		; there have to be two
	bne.s	bt_use_ret
	move.l	(a6,a1.l),d1	; get width and height into D1
	addq.l	#4,a1		; adjust stack
	bsr.s	ut_usnbt	; try to allocate in button frame
	bne.s	bt_use_ret

	move.l	d1,-(sp)	; store d1
	moveq	#6,d1		; check for 6 bytes on ri stack
	move.w	qa.resri,a1
	jsr	(a1)
	move.l	(sp)+,d1	; and restore d1
	move.l	bv_rip(a6),a1	; get ri stack pointer

	subq.l	#6,a1		; make room for float return parameter
	clr.w	(a6,a1.l)	; put zero exponent on
	tst.l	d1
	beq.s	ret_fl_mant	; ... and zero mantissa
	move.w	#$0820,d2	; and set unnormalised exponent (+1)
ret_fl_norm
	subq.w	#1,d2		; reduce exponent
	asl.l	#1,d1		; and multiply mantissa by 2
	bvc.s	ret_fl_norm	; if not overflowed yet, try again
	roxr.l	#1,d1		; restore mantissa to non overflowed
	move.w	d2,(a6,a1.l)	; put actual exponent on ri stack
ret_fl_mant
	move.l	d1,2(a6,a1.l)	; and mantissa
	moveq	#2,d4		; result is float
	move.l	a1,bv_rip(a6)
bt_use_ret
	rts

btframe_free
	bra.s	ut_frbtn	; just do the utility routine

;+++
; Find position in Button Frame for current job.
;
;		Entry				Exit
;	D1.l	width | height			x origin | y origin
;
;	Error returns: <> 0 if Button frame or THING does not exist
;---
ut_usnbt
btnu_reg reg	d2-d3/a0-a2
	movem.l btnu_reg,-(sp)
	moveq	#0,d3		; signal 'new entry'
	bra.s	use_btn
;+++
; Re-allocate a position in Button Frame for current job.
;
;		Entry				Exit
;	D1.l	width | height			x origin | y origin
;
;	Error returns: err.itnf 	Button frame does not exist
;---
ut_usrbt
	movem.l btnu_reg,-(sp)
	moveq	#-1,d3		; signal 're-allocate'
use_btn
	move.l	d1,d2
	moveq	#-1,d1		; for current job
	lea	btnf_nam,a0	; that's the Button Frame
	moveq	#sms.uthg,d0	; use it
	bsr.s	gu_thjmp
	tst.l	d0
	beq.s	use_ok		; failed, return default
use_err
	moveq	#-1,d2		; no thing, return default position
use_ok
	move.l	d2,d1		; that's the position!
	movem.l (sp)+,btnu_reg
	tst.l	d0
	rts

;+++
; Free entry in Button Frame.
;
; All registers including D0 are preserved.
;---
ut_frbtn
btnf_reg reg	d0-d3/a0-a2
	movem.l btnf_reg,-(sp)
	moveq	#-1,d1		; that's the current job
	lea	btnf_nam,a0	; the Thing we'd like to free
	moveq	#sms.fthg,d0	; free it
	bsr.s	gu_thjmp
	movem.l (sp)+,btnf_reg
	rts

btnf_nam
	dc.w	12
	dc.b	'Button Frame'

;+++
; Jump to Thing Utility through HOTKEY System II.
; Note this only works if a HOTKEY System version 2.03 or later is present.
;
;		Entry				Exit
;	d1	owner				Job ID
;	d2	priority/timeout		preserved
;	a0	thing name			preserved
;	a1	parameter string		preserved
;
;	Condition codes set
;---
gu_thjmp
	move.l	a4,-(sp)
	move.l	d0,-(sp)
	moveq	#thh_entr,d0	; thing vector required
	bsr.s	gu_thvec	; get THING vector
	bne.s	gut_ex4 	; there's nothing to jump to!
	move.l	(sp)+,d0
	jsr	(a4)		; do it
gut_exit
	move.l	(sp)+,a4
	tst.l	d0
	rts
gut_ex4
	addq.l	#4,sp		; skip operation
	bra.s	gut_exit

;+++
; Find Thing utilitiy vector of HOTKEY System II.
; Note this only works if a HOTKEY System version 2.03 or later is present.
;
;		Entry				Exit
;	d0	vector required 		error code
;	a4					Thing Utility Vector
;
;	Error returns:	err.nimp		THING does not exist
;	Condition codes set
;---
gu_thvec
	movem.l d1-d3/a0,-(sp)
	move.w	d0,d3
	moveq	#sms.info,d0	; get system variables
	trap	#do.smsq
	move.l	sys_thgl(a0),d1 ; this is the Thing list
	beq.s	thvec_nf	; empty list, very bad!
	move.l	d1,a0
thvec_lp
	move.l	(a0),d1 	; get next list entry
	beq.s	th_found	; end of list? Here should be THING!
	move.l	d1,a0		; next link
	bra	thvec_lp
thvec_nf
	moveq	#err.nimp,d0	; THING does not exist
	bra.s	thvec_rt
th_found
	move.l	th_thing(a0),a0 ; get start of Thing
	cmp.l	#-1,thh_type(a0); is it our special THING?
	bne.s	thvec_nf	; sorry, it isn't
	move.l	(a0,d3.w),a4	; this is the vector we look for
thvec_rt
	movem.l (sp)+,d1-d3/a0
	tst.l	d0
	rts

	end
